{-# LANGUAGE DerivingVia #-}
{-# LANGUAGE DeriveGeneric #-}
{-# LANGUAGE StandaloneKindSignatures #-}
{-# LANGUAGE DataKinds #-}
{-# LANGUAGE StandaloneDeriving #-}
{-# LANGUAGE FlexibleContexts #-}
{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE UndecidableInstances #-}
{-# LANGUAGE TypeFamilies #-}
module Language.Java.Classfile (Classfile(..)) where

import GHC.Generics (Generic, Generically(Generically))

import Data.Kind (Type)

import Language.Java.Classfile.Stage (Stage(Parse))
import Language.Java.Classfile.Version (Version)
import Language.Java.Classfile.Magic (Magic)
import Language.Java.Classfile.Extractable (Extractable)
import Language.Java.Classfile.ConstantPool (ConstantPool)
import Language.Java.Classfile.Flags (Flags)
import Language.Java.Classfile.ClassFlag (ClassFlag)
import Language.Java.Classfile.ConstantPool.References (ClassReference)
import Language.Java.Classfile.Interfaces (Interfaces)
import Language.Java.Classfile.Fields (Fields)
import Language.Java.Classfile.Methods (Methods)
import Language.Java.Classfile.Attributes (Attributes)


type Classfile :: Stage -> Type
data Classfile stage = Classfile
  { magic        :: Magic stage
  , version      :: Version
  , constantPool :: ConstantPool stage
  , accessFlags  :: ClassFlags stage
  , this         :: ClassReference
  , super        :: ClassReference
  , interfaces   :: Interfaces
  , fields       :: Fields
  , methods      :: Methods
  , attributes   :: Attributes
  }
  deriving stock (Generic)

deriving instance (Show (Magic stage), Show (ConstantPool stage), Show (ClassFlags stage)) => Show (Classfile stage)
deriving via Generically (Classfile Parse) instance (Extractable (Classfile Parse))

type ClassFlags :: Stage -> Type
data family ClassFlags stage

newtype instance ClassFlags Parse = ClassFlags (Flags ClassFlag)
  deriving stock (Show, Generic)
  deriving Extractable via Generically (ClassFlags Parse)
