{-# LANGUAGE GeneralizedNewtypeDeriving #-}
{-# LANGUAGE DataKinds #-}
{-# LANGUAGE DerivingVia #-}
{-# LANGUAGE TypeFamilies #-}
{-# LANGUAGE InstanceSigs #-}
{-# LANGUAGE LambdaCase #-}
module Language.Java.Classfile.ConstantPool.References (Utf8Reference(..), ClassReference(..), NameAndTypeReference(..), MethodHandleReferenceKind(..), OpaqueReference(..), BootstrapMethodIndex(..)) where
import Data.Word (Word16, Word8)
import Language.Java.Classfile.Extractable (Extractable)
import Language.Java.Classfile.Extractable.AsTag
    ( TagValue(..), AsTag, AsTag(..) )

newtype Utf8Reference = Utf8Reference Word16
  deriving stock (Show)
  deriving newtype Extractable

newtype ClassReference = ClassReference Word16
  deriving stock (Show)
  deriving newtype Extractable

newtype NameAndTypeReference = NameAndTypeReference Word16
  deriving stock (Show)
  deriving newtype Extractable

newtype OpaqueReference = OpaqueReference Word16
  deriving stock (Show)
  deriving newtype Extractable

newtype BootstrapMethodIndex = BootstrapMethodIndex Word16
  deriving stock (Show)
  deriving newtype Extractable

data MethodHandleReferenceKind
  = GetField
  | GetStatic
  | PutField
  | PutStatic
  | InvokeVirtual
  | InvokeStatic
  | InvokeSpecial
  | NewInvokeSpecial
  | InvokeInterface
  deriving stock (Show, Enum, Bounded)
  deriving Extractable via AsTag MethodHandleReferenceKind

instance TagValue MethodHandleReferenceKind where
  type TagType MethodHandleReferenceKind = Word8
  tagOf :: MethodHandleReferenceKind -> TagType MethodHandleReferenceKind
  tagOf = \case
    GetField         -> 1
    GetStatic        -> 2
    PutField         -> 3
    PutStatic        -> 4
    InvokeVirtual    -> 5
    InvokeStatic     -> 6
    InvokeSpecial    -> 7
    NewInvokeSpecial -> 8
    InvokeInterface  -> 9
